package com.persagy.dptool;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.persagy.dptool.annotation.FieldCheck;
import com.persagy.dptool.dto.PhysicalObject;
import com.persagy.dptool.dto.physical.PhysicalWorld;
import org.apache.http.client.config.RequestConfig;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.client.methods.HttpUriRequest;
import org.apache.http.entity.StringEntity;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClients;
import org.apache.http.impl.conn.PoolingHttpClientConnectionManager;
import org.apache.http.util.EntityUtils;

import java.io.*;
import java.lang.reflect.Field;
import java.net.URL;
import java.util.*;

public class CommonUtil {
    public static final ObjectMapper jsonMapper = new ObjectMapper();
    private final static PoolingHttpClientConnectionManager CONNECTION_MANAGER = new PoolingHttpClientConnectionManager();
    private final static CloseableHttpClient HTTP_CLIENT;
    private static final String UTF8 = "utf-8";
    public static String[] prefixStr = {"- Error", "- Warn"};

    static {
        // 设置整个连接池最大连接数 根据自己的场景决定
        CONNECTION_MANAGER.setMaxTotal(100);
        // 每个路由(网站)的最大连接数
        CONNECTION_MANAGER.setDefaultMaxPerRoute(50);
        RequestConfig requestConfig = RequestConfig.custom()
                // 与远程主机连接建立时间，三次握手完成时间
                .setConnectTimeout(60000)
                // 建立连接后，数据包传输过程中，两个数据包之间间隔的最大时间
                .setSocketTimeout(600000)
                // httpClient使用连接池来管理连接，这个时间就是从连接池获取连接的超时时间
                .setConnectionRequestTimeout(60000).build();
        HTTP_CLIENT = HttpClients.custom().setConnectionManager(CONNECTION_MANAGER)
                .setDefaultRequestConfig(requestConfig).build();
    }

    public static String getLineString(String str, int prefixIndex) {
        return  prefixStr[prefixIndex] + "：" + str + "\n";
    }

    private static String sendRequest(HttpUriRequest request) throws IOException {
        CloseableHttpResponse response = HTTP_CLIENT.execute(request);
        return EntityUtils.toString(response.getEntity(), UTF8);
    }

    public static String httpGetRequest(String url) throws Exception {
        HttpGet httpGet = new HttpGet(url);
        return sendRequest(httpGet);
    }

    public static String httpPostRequest(String url) throws Exception {
        HttpPost httpPost = new HttpPost(url);
        return sendRequest(httpPost);
    }

    public static byte[] httpGetFile(String url) throws Exception {
        HttpGet httpGet = new HttpGet(url);
        CloseableHttpResponse response = HTTP_CLIENT.execute(httpGet);
        return EntityUtils.toByteArray(response.getEntity());
    }

    public static String httpPostJson(String url, Map<Object, Object> params) throws IOException {
        HttpPost httpPost = new HttpPost(url);
        httpPost.addHeader("Content-Type", "application/json;charset=utf-8");
        httpPost.setEntity(new StringEntity(beanToJsonStr(params), UTF8));
        String respContent = sendRequest(httpPost);
        return respContent;
    }

    /**
     * 对象转json字符串
     * @param obj
     * @return
     * @throws JsonProcessingException
     */
    public static String beanToJsonStr(Object obj) throws JsonProcessingException {
        return jsonMapper.writeValueAsString(obj);
    }

    /**
     * json字符串转Map
     * @param jsonStr
     * @return
     * @throws IOException
     */
    public static Map<String, Object> jsonStrToMap(String jsonStr) throws IOException {
        return jsonMapper.readValue(jsonStr, new TypeReference<Map>() {});
    }

    /**
     * json字符串转对象
     * @param jsonStr
     * @param clz
     * @param <T>
     * @return
     * @throws IOException
     */
    public static <T> T jsonStrToObj(String jsonStr, Class<T> clz) throws IOException {
        return  jsonMapper.readValue(jsonStr, clz);
    }

    /**
     *
     * @param filePath resources下面的json文件路径， 例如：/vender/brand.json
     * @param clz
     * @param <T>
     * @return
     * @throws IOException
     */
    public static <T> T jsonStrToObjByResourceJsonFile(String filePath, Class<T> clz) throws IOException {
        URL rul = MainApp.class.getResource(filePath);
        return  jsonMapper.readValue(rul, clz);
    }

    /**
     * 读取properties文件信息，转为map
     * @param propertyFile
     * @return
     */
    public static Map<String, String> property2Map(File propertyFile) {
        Map<String, String> result = new HashMap<>();

        Properties prop = new Properties();
        try(InputStream in = new FileInputStream(propertyFile)){
            prop.load(in);
            Iterator<String> it = prop.stringPropertyNames().iterator();
            String key = null;
            while(it.hasNext()){
                key = it.next();
                result.put(key, prop.getProperty(key));
            }
        } catch (Exception e) {
            e.printStackTrace();
        }

        return result;
    }

    /**
     * 按行读取文件中的信息，将所读到信息写入到一个字符串当中
     * @param file 文件
     * @return String 文件中的信息
     */
    public static String readFileToString(File file) {
        StringBuffer sbStr = new StringBuffer();
        List<String> dataStringList = readFileToList(file);
        if(null != dataStringList) {
            for(String str : dataStringList) {
                sbStr.append(str);
            }
        }

        return sbStr.toString();
    }

    /**
     * 按行读取文件中的信息存入到List集合中
     * @param file
     * @return List<String> 文件中的内容，按行存入集合，默认返回空集合
     */
    public static List<String> readFileToList(File file) {
        List<String> contentList = new ArrayList<>();

        if(null == file || !file.exists() || !file.isFile()) {
            return contentList;
        }

        try (InputStreamReader isr = new InputStreamReader(new FileInputStream(file));
             BufferedReader br = new BufferedReader(isr);) {
            String temp = null;
            while ((temp = br.readLine()) != null) {
                contentList.add(temp);
            }
        } catch (Exception e) {
            e.printStackTrace();
        }

        return contentList;
    }

    /**
     * 根据朱姐校验物理世界对象属性值
     * @param objects
     * @param <T>
     * @param physicalObject
     * @return 包含Error字符的错误详情信息或者null
     */
    public static <T extends PhysicalWorld> String annotationFieldCheck(List<T> objects, PhysicalObject physicalObject) {
        if(null == objects || objects.isEmpty()) {
            return null;
        }

        StringBuilder sbResultStr = new StringBuilder();

        PhysicalWorld example = objects.get(0);
        Class clazz = example.getClass();

        Map<String, Set<Object>> fieldName2UniqValue = new HashMap<>();
        for (Field field : clazz.getDeclaredFields()) {
            if (field.isAnnotationPresent(FieldCheck.class) && field.getAnnotation(FieldCheck.class).uniq()) {
                fieldName2UniqValue.put(field.getName(), new HashSet<>());
            }
        }

        for(Object obj : objects) {
            try {
                for (Field field : clazz.getDeclaredFields()) {
                    if (field.isAnnotationPresent(FieldCheck.class)) {
                        String errorMsg = "";
                        String fieldName = field.getName();

                        FieldCheck fieldCheck = field.getAnnotation(FieldCheck.class);
                        field.setAccessible(true);
                        Object value = field.get(obj);

                        if (!fieldCheck.canBeNull() && null == value) {
                            errorMsg += errorTip(fieldName,ConstantData.errorCodes[1]);
                        }

                        if (fieldCheck.length() > 0 && null != value && value.toString().length() != fieldCheck.length()) {
                            errorMsg += errorTip(fieldName,ConstantData.errorCodes[2]);
                        }

                        if(fieldCheck.uniq()) {
                            if(fieldName2UniqValue.containsKey(fieldName) && fieldName2UniqValue.get(fieldName).contains(value)) {
                                errorMsg += errorTip(fieldName,ConstantData.errorCodes[4]);
                            }else {
                                fieldName2UniqValue.get(fieldName).add(value);
                            }
                        }

                        if(fieldCheck.existObj() && physicalObject.objIdSet != null && !physicalObject.objIdSet.contains(value)) {
                            errorMsg += errorTip(fieldName,ConstantData.errorCodes[3]) + "相关表:objects; ";
                        }

                        if(fieldCheck.existGraphId() && physicalObject.graphIdSet != null && !physicalObject.graphIdSet.contains(value)) {
                            errorMsg += errorTip(fieldName,ConstantData.errorCodes[3]) + "相关表:graph_instance; ";
                        }

                        if(fieldCheck.existInfoCodeSchemeId() && physicalObject.infoCodeSchemeIdSet != null && !physicalObject.infoCodeSchemeIdSet.contains(value)) {
                            errorMsg += errorTip(fieldName,ConstantData.errorCodes[3]) + " 相关表:infocode_scheme; ";
                        }

                        if(fieldCheck.existCtmDictId() && physicalObject.ctmDicIdSet != null && !physicalObject.ctmDicIdSet.contains(value)) {
                            errorMsg += errorTip(fieldName,ConstantData.errorCodes[3]) + "相关表:ctm_dict; ";
                        }

                        if(fieldCheck.existCtmDictNodeType() && !ConstantData.customDictNodeType.contains(value)) {
                            errorMsg += fieldName + ":" + ConstantData.errorCodes[2] + "值范围:" + ConstantData.customDictNodeType;
                        }

                        if(errorMsg.length() > 0) {
                            sbResultStr.append("异常记录：" + obj.toString() + errorMsg + "\n");
                        }
                    }
                }

            }catch (Exception e) {
                e.printStackTrace();
            }
        }

        if(sbResultStr.length() > 0) {
            return sbResultStr.toString();
        }

        return null;
    }

    private static String errorTip(String fieldName, String errorCode) {
        return "异常字段:" + fieldName + ", 错误码:" + errorCode + ", ";
    }

    /**
     * 测试数据平台服务连通性
     * @param serviceUrl 数据平台地址 http://ip:port/data-platform-3
     * @return
     */
    public static boolean dataPlatformConnectTest(String serviceUrl) {
        boolean result = false;

        String url = serviceUrl + "/dict/query/direction";
        try {
            String respStr = httpGetRequest(url);
            if(null != respStr) {
                result = respStr.contains("success");
            }
        }catch (Exception e){}

        return result;
    }

    public static void main(String[] args) {
        List<String> dataList = readFileToList(new File("D:/downloads/DingDing/download/4403050019/objects.json"));
        Set<String> allKey = new HashSet<>();
        Set<String> repeat = new HashSet<>();
        try {
            for(String dataStr : dataList) {
                Map<String, Object> mapData = jsonStrToMap(dataStr);
                String obj_idAndinfo_id = mapData.get("obj_id") + "";
                if(allKey.contains(obj_idAndinfo_id)) {
                    repeat.add(obj_idAndinfo_id);
                }else {
                    allKey.add(obj_idAndinfo_id);
                }
            }
        }catch (Exception e){}

        System.out.println(allKey.size());
        System.out.println(repeat);
    }
}
